/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ARGS.h"


ARGS::ARGS ( const int argc, char *Argv[] )
{
	string s;
	for (int i=argc-1; i>0; i--){
		s.assign(Argv[i]);
		argv.push(s);
	}
	
	OptList.clear();
	for (int i=0; i<OptCount; i++) OptList.insert(make_pair(Opt[i], i));
}

int ARGS::Make()
{
	bool u = false, ul = false, v = false, all = false, good_number;
	bool ip = false, p = false, dl = false, n = false, del = false, st = false;
	bool debug = Debug();
	
	string arg, msg;
	int val, num;
	IP tmp_ip;
	
	User tmp_user, user, stat;
	ActiveUser tmp_active;
	Stime tm1, tm2;
	DataBase db;
		
	vector<string>::iterator i;
	
	while ( !argv.empty() ){
		
		arg = argv.top();
		argv.pop();
		if ( OptList.find(arg) == OptList.end() ){
			msg = arg + string( " - illegal option" );
			if ( debug ) WriteLog( msg );
			Report(Error, msg);
			return 4;
		} 
		val = OptList[arg];
		
		switch (val){
			case 0: //"-D"
				SetDebugMode( 1 );
				break;
			case 1: //"-pl"
				msg = string( "Setting IP-POOL: " ) + argv.top();
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				
				switch( SetIpPool( argv.top() ) ){
					case 0:
						msg = "IP-POOL was succesfully set";
						if ( debug ) WriteLog( msg );
						Report( Info, msg );
						break;
					case 2:
						msg = "writing IP_POOL file failed";
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 2;
						break;
					case 4:
						msg = "wrong ip range format(example: 10.8.0.1-10.8.0.32 )";
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 4;
						break;
					case 5:
						msg = "first ip is illegal";
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 4;
						break;
					case 6:
						msg = "second ip is illegal";
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 4;
						break;
					case 7:
						msg = "first IP is greater then second";
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 4;
						break;
				}
				argv.pop();
				break;
				
			case 2: // -c
				switch ( UpdateIfaces() ){
					case 0:
						msg = "All connections were succesfully checked";
						if ( debug ) WriteLog( msg );
						Report( Info, msg );
						break;
					case 5:
						msg = string( "Unable to read proc file " ) + string( FILE_PROC );
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 1;
						break;
					case 6:
						msg = string( "Unable to read ifaces file: " ) 
								+ string( DIR_PROGRAM ) + string( FILE_IFACES );
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 1;
						break;
				}
				break;
				
			case 3: // -u
				//Checking preveasly options -u
				if (ul || u) {
					msg = string( "using -u or -ul combinations is illegal" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
			
				msg = "Choosing user";
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				
				u = true;
				user.login = argv.top();
				argv.pop();
				
				msg = string("Login: ") + user.login;
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				break;
				
			case 4: // -n
				if ( !u ){
					msg = "No user has been choosen";
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				
				user.name = argv.top();
				msg = string("Name: ") + user.name;
				n = true;
				
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				argv.pop();
				break;
			
			case 5: // -ip
				if ( !u ) {
					msg = "No user has been choosen";
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				
				if ( !user.ip.Assign( argv.top() ) ){
					msg = argv.top() + string( " is illegal ip" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				argv.pop();
				ip = true;
				
				msg = string( "IP: " ) + user.ip.Str();
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				break;
			
 			case 6: // -p
				if ( !u ) {
					msg = "No user has been choosen";
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
						
				user.pass = argv.top();
				p = true;
				
				msg = string("Password: ") + user.pass;
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				argv.pop();
				break;
				
			case 7: // -dl
				if ( !u ) {
					msg = "No user has been choosen";
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				
				good_number = true;
				num = 0;
				for (size_t i=0; i<argv.top().length(); i++)
					if ( isdigit(argv.top()[i]) ) num = num*10 + (argv.top()[i]-'0'); 
						else{
							good_number = false;
							break;
						}
						
				if ( !good_number && argv.top() != "-1" ){
					msg = argv.top() + string( " - is illegal date limit");
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;	
				}
				if (argv.top() == "-1") num = -1;
				if ( num != -1 ) user.d_limit = num * 1024 *1024; else user.d_limit = num;
				dl = true;
				
				msg = string("Day limit: ") + argv.top();
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				argv.pop();
				break;
				
			case 11: // -del
				if ( !u ) {
					msg = "No user has been choosen";
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				
				if ( db.FindUser( user.login, tmp_user ) == 5 ){
					msg = string( "No user with login: " ) + user.login;
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				} 
				
				switch ( db.DeleteUser( user.login ) ){
					case 1:
						msg = string( "unable to read users file:" ) + string( DIR_PROGRAM ) 
								+ string( FILE_USERS );
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 1;
						break;
					case 2:
						msg = string( "unable to write users file:" ) + string( DIR_PROGRAM ) 
								+ string( FILE_USERS );
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 2;
						break;
				}
				del = true;
				
				msg = "User was succesfully deleted";
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				break;
				
			case 12: // -ls
				ReportUsersList();
				break;
							
			case 13: // -en
				if ( !tmp_ip.Assign( argv.top() ) ){
					msg = argv.top() + string( " is illegal ip" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				argv.pop();
				
				if ( db.FindUser( tmp_ip, tmp_user ) == 5 ){
					msg = string( "No user with IP: " ) + tmp_ip.Str();
					if ( debug ) WriteLog( msg );
					Report( Info, msg );
					return 4;
				}
				
				tmp_active.ip = tmp_ip;
				tmp_active.rx = 0;
				tmp_active.tx = 0;
				db.InsertActive( tmp_active ); 
				
				
				msg = string( "User with IP: " ) + tmp_ip.Str() + string( " was sucessfully activated" );
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				break;
			
			case 14: // -dis
				if ( !tmp_ip.Assign( argv.top() ) ){
					msg = argv.top() + string( " is illegal ip" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				argv.pop();
								
				if ( db.FindUser( tmp_ip, tmp_user ) == 5 ){
					msg = string( "No user with IP: " ) + tmp_ip.Str();
					if ( debug ) WriteLog( msg );
					Report( Info, msg );
					return 4;
				}
				
				db.DeleteActive( tmp_ip );
				 
				msg = string( "User with IP: " ) + tmp_ip.Str() + string( " was sucessfully deactivated" );
				if ( debug ) WriteLog( msg );
				Report( Info, msg );
				break;
				
			case 8: // -t
				if ( !AssignPeriod( argv.top(), tm1, tm2 ) ){
					msg = argv.top() + string( " is illegal time period" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				
				if ( u ){
					if ( ReportByDayPersTraffic( tm1, tm2, user.login, cout ) == 5 ){
						msg = string( "No user with login: " ) + user.login + string( " found." );
						if ( debug ) WriteLog( msg );
						Report( Info, msg );
					}
					u = false;
				} else {
					ReportPeriodAllTraffic( tm1, tm2, cout );	
				}
				
				argv.pop();
				break;
			
			case 9: // -d
				if ( u ){
					tm1.Refresh();
					if ( ReportByDayPersTraffic( tm1, tm1, user.login, cout ) == 5 ){
						msg = string( "No user with login: " ) + user.login + string( " found." );
						if ( debug ) WriteLog( msg );
						Report( Info, msg );
					}
					u = false;
				} else {
					tm1.Refresh();
					ReportPeriodAllTraffic( tm1, tm1, cout ); 
				}	
				break;
				
			case 10: // -m
				if ( u ){
					tm1.Refresh();	tm1.SetD( 1 );
					tm2 = tm1;		tm2.SetD( 31 );
					if ( ReportByDayPersTraffic( tm1, tm2, user.login, cout ) == 5 ){
						msg = string( "No user with login: " ) + user.login + string( " found." );
						if ( debug ) WriteLog( msg );
						Report( Info, msg );
					}
					u = false;
				} else {
					tm1.Refresh();	tm1.SetD( 1 );
					tm2 = tm1;		tm2.SetD( 31 );
					ReportPeriodAllTraffic( tm1, tm2, cout );
				}
				break;
				
			case 16: // -per
				//Outputs all users day traffic for every day in period
				if ( !AssignPeriod( argv.top(), tm1, tm2 ) ){
					msg = argv.top() + string( " is illegal time period" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
				ReportByDayAllTraffic( tm1, tm2, cout );
							
				argv.pop();
				break;
				
			case 15: // -act
				ReportActiveList();
				break;
				
		}
	}
	
	if (u && !del){
				
		int found = db.FindUser( user.login, tmp_user );
		
		
		if ( found == 5 ) {	
			if (!n) user.name = "user";
			if (!p) user.pass = "11111";
			if (!dl) user.d_limit = -1;
			if (!ip){
				switch ( user.ip.GetFromPool() ){
					case 5:
						msg = "no free ips in pool";
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 4;
						break;
					case 6:
						msg = string ( "reading ip-pool file " ) + string( DIR_PROGRAM )
								+ string( FILE_IP_POOL ) + string( "failed" );
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 1;
					break;
					case 7:
						msg = string ( "reading user file " )
								+ string( DIR_PROGRAM ) + string( FILE_IP_POOL ) + string( "failed" );
						if ( debug ) WriteLog( msg );
						Report( Error, msg );
						return 1;
						break;
				}
			}
			//if ( db.FindUser( user.login, tmp_user ) != 5 )	db.DeleteUser( user.login );
		
			switch ( db.InsertUser( user ) ){
				case 1:
					msg = string ( "reading user file " )
							+ string( DIR_PROGRAM ) + string( FILE_IP_POOL ) + string( "failed" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 1;
					break;
				case 2:
					msg = string ( "writing user file " )
							+ string( DIR_PROGRAM ) + string( FILE_IP_POOL ) + string( "failed" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 2;
					break;
			}
		
			msg = "user was successfully inserted";
			if ( debug ) WriteLog( msg );
			Report( Info, msg );
			
		} else {//if user was found
			if (!ip) user.ip = tmp_user.ip; else{
				if ( db.FindUser( user.ip, tmp_user ) != 5 ){
					msg = string( "user with ip: " ) + user.ip.Str() + string( " already exist" );
					if ( debug ) WriteLog( msg );
					Report( Error, msg );
					return 4;
				}
			}
			if (!p) user.pass = tmp_user.pass;
			if (!n) user.name = tmp_user.name;
			if (!dl) user.d_limit = tmp_user.d_limit;
			db.DeleteUser( user.login );
			db.InsertUser( user );
				
			msg = "user was successfully edited";
			if ( debug ) WriteLog( msg );
			Report( Info, msg );
		}
	}
		 
	return 0;							
}

