/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Check.h"

#include <iostream>
#include <signal.h>
#include <sstream>
#include <fstream>
#include <string>
#include <vector>
#include <map>

int Iface::GetPid()
{
	ifstream pid_file;
	string fname = DIR_PID;
	fname += name;
	fname += ".pid";
	
	pid_file.open(fname.c_str());
	if ( !pid_file ) return 1;
	
	pid_file >> pid;
	
	return 0;
}

int Kill( long pid )
{
	return kill( pid, SIGKILL );
}

int UpdateIfaces()
{	
	bool debug = Debug();
	
	//get proc info
	string s;
	Iface tmp;
	ifstream proc;
	istringstream ss;
	map<string, Iface> iface;
	
	proc.open(FILE_PROC, ios_base::in);	
	if ( !proc ) return 5;
	
	getline( proc, s );
	getline( proc, s );
	while ( getline(proc, s) ){
		tmp.name = s.substr( 0, s.find( ":" ) );
		while ( tmp.name[0] == ' ' ) tmp.name.erase(0, 1);
		s.erase(0, s.find(":")+1);
		ss.clear();
		ss.str( s );
		ss >> tmp.rx;
		ss >> tmp.tx >> tmp.tx >> tmp.tx >> tmp.tx >> tmp.tx >> tmp.tx >> tmp.tx >> tmp.tx;
		iface[tmp.name] = tmp;
	}
	
	proc.close();	
	
	// verify connected users from "ifconfig | grep "P-t-P" -B1"
	string loc(DIR_PROGRAM);
	loc += FILE_IFACES;
	
	ifstream file_ifaces;
	file_ifaces.open( loc.c_str(), ios_base::in );
	
	if ( !file_ifaces ) return 6;
	
	while ( !file_ifaces.eof() ){
		
		file_ifaces >> tmp.name;
		getline(file_ifaces, s);
		
		file_ifaces >> s >> s >> s;
		s.erase(0, 8-2); //"8-2" means delete "P-t-P:"
		tmp.ip.Assign( s );
		getline(file_ifaces, s);
		getline(file_ifaces, s);
		WriteLog( tmp.name );
		  
		iface[tmp.name].ip = tmp.ip;
	}
	file_ifaces.close();
	
	//Active and connected
	DataBase db;
	vector<ActiveUser> active;
	db.GetActiveList( active );
	
	//Checking all active
	Stime tm;
	bool found;
	UserStat US;
	typedef vector<ActiveUser>::iterator VAUI;
	typedef map<string, Iface>::iterator MSII;
	
	tm.Refresh();
	for (VAUI i=active.begin(); i!=active.end(); i++){
		//Verify active
		if ( debug ) WriteLog( "Updating user info, ip:" + i->ip.Str() );
		found = false;
		for (MSII j=iface.begin(); j!=iface.end(); j++)
			if (j->second.ip == i->ip) {found = true; tmp = j->second; break;}
		
		if (found) {
			if ( debug ) WriteLog( "This user is connected. Updating session trafic." );
			i->rx = tmp.rx;
			i->tx = tmp.tx;
			
			// Disconect if over limit 
			User user;
			db.FindUser( i->ip, user );
			db.GetStat( tm, i->ip, US );
			if ( US.rx + i->rx + US.tx + i->tx > user.d_limit && user.d_limit != -1 ){
				tmp.GetPid();
				Kill( tmp.pid );
				US.rx += i->rx;
				US.tx += i->tx;
				db.InsertStat( tm, US );
				WriteLog( "Deleteing active that over limit" );
				db.DeleteActive( i->ip );
				WriteLog( string( "Disconnecting user with ip: " ) + i->ip.Str() );
			}
			
			db.InsertActive( *i );
		} else {
			if ( debug ) WriteLog( "This user is not connected. Inserting session trafic to journal." );
			db.GetStat( tm, i->ip, US );
			US.rx += i->rx;
			US.tx += i->tx;
			db.InsertStat( tm, US );
			db.DeleteActive( i->ip );
		}
	}
	
	return 0;
}
