/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <fstream>
#include <string>
#include <vector>

#include "DataBase.h"

using namespace std;

/* Insert and Delete user */
int DataBase::InsertUser( const User& user )
{
	string fname = DIR_PROGRAM;
	fname += FILE_USERS;
	
	UserContainer<User> UC;
		
	if (UC.Load( fname ) == 1 ) return 1;
	UC.Insert( user );
	if (UC.Store( fname ) == 2 ) return 2;
	
	DeleteFromChap( user );
	InsertToChap( user );
	
	return 0;
}

int DataBase::DeleteUser( const string& login )
{
	string fname = DIR_PROGRAM;
	fname+= FILE_USERS;
	
	UserContainer<User> UC;
	
	if ( UC.Load( fname ) == 1 ) return 1;
	
	size_t i;
	for (i=0; i<UC.Size(); i++)
		if (UC[i].login == login) break;
	UC.Delete( i );
	
	if ( UC.Store( fname ) == 2 ) return 2;
	
	User tmp;
	tmp.login = login;
	DeleteFromChap( tmp );
	
	return 0;
}

int DataBase::DeleteUser( const IP& ip )
{
	string fname = DIR_PROGRAM;
	fname+= FILE_USERS;
	
	UserContainer<User> UC;
	
	if ( UC.Load( fname ) == 1 ) return 1;
	
	size_t i;
	for (i=0; i<UC.Size(); i++)
		if (UC[i].ip == ip) break;
	UC.Delete( i );
	
	if ( UC.Store( fname ) == 2 ) return 2;
	
	User tmp;
	FindUser( ip, tmp );
	DeleteFromChap( tmp );
	
	return 0;
}


/* Insert and Delete active user */
int DataBase::InsertActive( const ActiveUser& user )
{
	string fname = DIR_PROGRAM;
	fname += FILE_ACTIVE;
	
	UserContainer<ActiveUser> UC;
	
	if ( UC.Load( fname ) == 1 ) return 1;
	
	UC.Insert( user );
	
	if ( UC.Store( fname ) == 2 ) return 2;
	
	return 0;
}

int DataBase::DeleteActive( const IP& ip )
{
	string fname = DIR_PROGRAM;
	fname+= FILE_ACTIVE;
	
	UserContainer<ActiveUser> UC;

	if ( UC.Load( fname) == 1 ) return 1;
	
	size_t i;
	for (i=0; i<UC.Size(); i++)
		if (UC[i].ip == ip) break;
	UC.Delete( i );
	
	if ( UC.Store( fname ) == 2) return 2;
	
	return 0;
}

int DataBase::GetActiveList( vector<ActiveUser>& users )
{
	string fname = DIR_PROGRAM;
	fname += FILE_ACTIVE;
	
	UserContainer<ActiveUser> UC;
	
	if ( UC.Load( fname ) == 1 ) return 1;
	
	size_t i;
	users.clear();
	for (i=0; i<UC.Size(); i++)
		users.push_back( UC[i] );
	
	return 0;
}

int DataBase::GetUserList( vector<User>& users )
{
	string fname = DIR_PROGRAM;
	fname += FILE_USERS;
	
	UserContainer<User> UC;
	
	if ( UC.Load( fname ) == 1 ) return 1;
	
	size_t i;
	users.clear();
	for (i=0; i<UC.Size(); i++)
		users.push_back( UC[i] );
	
	return 0;	
}


/* Add and Get statistic */
int DataBase::InsertStat( const Stime& t, const UserStat& st )
{
	string loc(DIR_PROGRAM);
	loc += DIR_JOURNAL;
	loc += t.Str();
	 
	UserContainer<UserStat> UC;
	
	if ( OpenForReading( loc ) == 1 ) {
		if ( CreateFile( loc, "0" ) == 2 ) return 2;
	}
	
	if ( UC.Load( loc ) == 1 ) return 1;
	UC.Insert( st );
	if ( UC.Store( loc ) == 2 ) return 2;
	
	return 0;
}
 
int DataBase::GetStat( const Stime& t, const IP& ip, UserStat& st )
{
	string loc(DIR_PROGRAM);
	loc += DIR_JOURNAL;
	loc += t.Str();
	 
	UserContainer<UserStat> UC;
	
	st.ip = ip;	st.rx = 0; st.tx = 0;
		
	if ( UC.Load( loc ) == 1 ) return 1;
	for (int i=0; i<UC.Size(); i++)
		if (UC[i].ip == ip){
			st = UC[i];
			return 0;
		}
		
	return 5;
} 
 
/* Find User and Active */
int DataBase::FindUser( const IP& ip, User& user )
{
	string loc = DIR_PROGRAM;
	loc += FILE_USERS;
	
	UserContainer<User> UC;
	if ( UC.Load( loc ) == 1 ) return 1;
	return UC.Find( ip, user );
}

int DataBase::FindUser( const string& login, User& user )
{
	string loc = DIR_PROGRAM;
	loc += FILE_USERS;
	
	UserContainer<User> UC;
	if ( UC.Load( loc ) == 1 ) return 1;
	return UC.Find( login, user );
}

int DataBase::FindActive( const IP& ip, ActiveUser& user )
{
	string loc = DIR_PROGRAM;
	loc += FILE_ACTIVE;
	
	UserContainer<ActiveUser> UC;
	if ( UC.Load( loc ) == 1 ) return 1;
	return UC.Find( ip, user );
}
