/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "IP.h"
#include "defs.h"
#include "DataBase.h"

#include <cctype>
#include <sstream>
#include <fstream>
#include <iostream>

string IP::Str() const
{
	ostringstream ss;
	ss << a << "." << b << "." << c << "." << d;
	return ss.str();
}

bool IP::Assign( string s )
{
	
	string t;
	int num;
	
	for (int i=0; i<3; i++){
		
		if (s.find('.') == string::npos) return false;
		
		t = s.substr(0, s.find('.'));
		//cout << "t = " << t << endl;
		if (t.length()>3 || t.length() == 0) return false;
		
		num = 0;
		for (size_t j=0; j<t.length(); j++)
			//IP can't be so 192.168.014.8
			if (j == 0 && t[j] == '0' && t.length() > 1) return false; else
			if ( !isdigit( t[j] ) ) return false; else num = num*10 + (t[j] - '0');
		
		if (num<0 || num>255) return false;
		
		if (i==0) a = num; else
		if (i==1) b = num; else
		if (i==2) c = num;			 
		
		s.erase(0, s.find('.') + 1);
		//cout << "s = " << s << endl;
	}
	
	if (s.length()>3 || s.length() == 0) return false;
		
	num = 0;
	for (size_t j=0; j<s.length(); j++)
		if (j == 0 && s[j] == '0' && s.length() > 1) return false; else
		if ( !isdigit( s[j] ) ) return false; else num = num*10 + (s[j] - '0');
	
	if (num<0 || num>255) return false;
	d=num;
	
	return true;
	
}

int SetIpPool( string& pool )
{
	int pos = pool.find("-");
	if (pos == string::npos) return 4;
	
	IP ip1, ip2;
	string s;
	
	s = pool.substr(0, pos);
	if ( !ip1.Assign( s ) ) return 5;
	s = pool.substr(pos+1, pool.length()-pos-1);
	if ( !ip2.Assign( s ) ) return 6;
	
	if (ip1 > ip2) return 7;
	
	ofstream out;
	s = DIR_PROGRAM;
	s += FILE_IP_POOL;
	out.open(s.c_str(), ios_base::out);
	if (!out) return 2;
	
	out << ip1.Str() << endl << ip2.Str() << endl;
	
	out.close();
	return 0;
}

int GetIpPool(IP& beg, IP& end)
{
	string loc = DIR_PROGRAM;
	loc += FILE_IP_POOL;
	
	ifstream in;
	in.open(loc.c_str(), ios_base::in);
	
	if (!in) return 1;
	
	string ip_str;
	getline(in, ip_str);
	beg.Assign( ip_str );
	getline(in, ip_str);
	end.Assign( ip_str );
	
	in.close();
	return 0;
}

int IP::GetFromPool()
{
	IP beg, end;
	DataBase db;
	User user;
	
	if ( GetIpPool( beg, end ) == 1) return 6;
	
	int res;
	bool found = false;
	for (IP i=beg; i<=end; ++i){
		res = db.FindUser( i, user );
		if ( res == 1 ) return 7;
		if ( res == 5 ) {*this = i; return 0;}
	}
	return 5;
}

IP& IP::operator ++()
{
	d++;
	if (d == 256) {c++; d=0;}
	if (c == 256) {b++; c=0;}
	if (b == 256) {a++; b=0;}
	if (a == 256) a=0;
	
	return *this;
}

bool IP::operator== ( const IP& ip )
{
	return ( (a == ip.a) && (b == ip.b) && (c == ip.c) && (d == ip.d) );
}

IP& IP::operator= ( const IP& ip )
{
	a = ip.a;
	b = ip.b;
	c = ip.c;
	d = ip.d;
}

bool IP::operator <( const IP& ip )
{
	if (a<ip.a) return true; else
	if (a>ip.a) return false; else
				if (b<ip.b) return true; else
				if (b>ip.b) return false; else
							if (c<ip.c) return true; else
							if (c>ip.c) return false; else
										if (d<ip.d) return true; else return false;
					
}

bool IP::operator >( const IP& ip )
{
	return (!operator<(ip) && !operator==(ip));
}

bool IP::operator <=( const IP& ip )
{
	return (operator<(ip) || operator==(ip));
}

bool IP::operator >=( const IP& ip )
{
	return (operator>(ip) || operator==(ip));
}
