/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Report.h"

void UserStrings(User& user, vector<string>& Word)
{
	Word.clear();
	Word.push_back( user.name );
	Word.push_back( user.ip.Str() );
	Word.push_back( user.login );
	Word.push_back( user.pass );
	
	ostringstream ss;
	
	ss << setiosflags(ios::fixed);
	double d = (double)user.d_limit/1024.0/1024.0;
		
	if ( user.d_limit != -1 ) ss << setprecision(2) << d << " MB"; else ss << "Unlimit";
	Word.push_back( ss.str() );
}

void ActiveStrings(ActiveUser& user, vector<string>& Word)
{
	DataBase db;
	User tmp_user;
	
	db.FindUser( user.ip, tmp_user );
	
	Word.clear();
	Word.push_back( tmp_user.name );
	
	ostringstream ss;
	double d;
	
	ss << setiosflags(ios::fixed);
	d = (double)user.tx/1024.0/1024.0;
	ss << setprecision(2) << d << " MB";
	Word.push_back( ss.str() );
	ss.str("");
	d = (double)user.rx/1024.0/1024.0;
	ss << setprecision(2) << d << " MB";
	Word.push_back( ss.str() );
}

void StatStrings( UserStat& st, vector<string>& Word )
{	
	DataBase db;
	User user;
	
	Word.clear();
	if ( db.FindUser( st.ip, user ) == 5 ) user.name = "TOTAL"; 
	
	Word.push_back( user.name );
	ostringstream ss;
	double d;
	
	ss << setiosflags(ios::fixed);
	d = (double)st.tx/1024.0/1024.0;
	ss << setprecision(2) << d << " MB";
	Word.push_back( ss.str() );
	ss.str("");
	d = (double)st.rx/1024.0/1024.0;
	ss << setprecision(2) << d << " MB";
	Word.push_back( ss.str() );
}

void DTStrings( DayTraffic& dt, vector<string>& Word )
{
	Word.clear();
	//10.10.9998 is used to say that dt containes TOTAL trafic 
	if ( dt.tm.Str() != "10.10.9998" ) Word.push_back( dt.tm.Str() ); else Word.push_back( "TOTAL" );
	
	ostringstream ss;
	double d;
	
	ss << setiosflags(ios::fixed);
	d = (double)dt.tx/1024.0/1024.0;
	ss << setprecision(2) << d << " MB";
	Word.push_back( ss.str() );
	ss.str("");
	d = (double)dt.rx/1024.0/1024.0;
	ss << setprecision(2) << d << " MB";
	Word.push_back( ss.str() );
}

int Report( ReportData RD, const string& msg, ostream& out )
{
	switch (RD) {
		case Info:
			out << msg << endl;
			break;
		case Error:
			out << "Error: " << msg << endl;
			break;
	}
	return 0;
}

int ReportUsersList( ostream& out )
{
	DataBase db;
	vector<User> users;
	vector<string> Label;
	Table<User> tb(UserStrings, &out);
	
	db.GetUserList( users );
	if ( users.size() == 0 ) { out << "No users in database" << endl; return 0; }
	
	tb.CalcColWidth( users );
	Label.push_back( "Name" );	Label.push_back( "IP" );	Label.push_back( "Login" );
	Label.push_back( "Password" );	Label.push_back( "Day limit" );
	tb.SetLabels( Label );
	tb.DrawLabels();
	
	for (size_t i=0; i<users.size(); i++)
		tb.DispRow(users[i]);
	
	return 0;
}

int ReportActiveList( ostream& out )
{
	DataBase db;
	vector<ActiveUser> users;
	vector<string> Label;
	Table<ActiveUser> tb(ActiveStrings, &out);
	
	db.GetActiveList( users );
	if ( users.size() == 0 ){ 
		out << "No active in database" << endl;
		return 1;
	}
		
	
	tb.CalcColWidth( users );
	Label.push_back( "IP" ); Label.push_back( "Tx" );	Label.push_back( "Rx" );
	tb.SetLabels( Label );
	tb.DrawLabels();
	
	for (size_t i=0; i<users.size(); i++)
		tb.DispRow(users[i]);
	
	return 0;
}

int ReportByDayPersTraffic( Stime& tm1, Stime& tm2, string& login, ostream& out )
{
	/* Report users day trafic for every day in PERIOD */
	DataBase db;
		
	UserStat tmp_stat, sum;
	User user;
	
	vector<DayTraffic> stat;
	DayTraffic dt;
	
	int res; //to store functions result value
	
	sum.rx = 0; sum.tx = 0;
	if ( db.FindUser( login, user ) == 5) return 5;
	
	sum.ip = user.ip;
	
	vector<string> Label;
	Table<DayTraffic> tb( DTStrings, &out );
	
	stat.clear();
	//10.10.2008 is used to calculate usual date column width
	dt.tm.Assign( "10.10.2008" );
	dt.rx = LONG_MAX;
	dt.tx = LONG_MAX;
	stat.push_back( dt );
	
	Report( Info, string( "Traffic for PERIOD: " ) + tm1.Str() + string( " - " ) + tm2.Str() );  
		
	tb.CalcColWidth( stat );
	Label.push_back( "Day" ); Label.push_back( "Tx" );	Label.push_back( "Rx" );
	tb.SetLabels( Label );
	tb.DrawLabels();
	
	for (Stime i=tm1; i<=tm2; ++i){
		/* We can output all day, or only days on which statistic is found*/
		res = db.GetStat( i, sum.ip, tmp_stat);
		if ( res != 5 && res != 1 ){
			sum.tx += tmp_stat.tx;
			sum.rx += tmp_stat.rx;
			
			dt.tm = i;
			dt.rx = tmp_stat.rx;
			dt.tx = tmp_stat.tx;
			tb.DispRow( dt );
		}
	}
	
	//MAX_YEAR
	if ( tm1 != tm2){
		//Program shouldn't output "TOTAL" for -u -d option
		dt.tm.Assign( "10.10.9998" );
		dt.rx = sum.rx;
		dt.tx = sum.tx;
		tb.DispRow( dt );
	}
	
	return 0;
}

int ReportByDayAllTraffic( Stime& tm1, Stime& tm2, ostream& out )
{
	//Outputing table headline
	Report( Info, string( "Traffic for all users PERIOD: " ) + tm1.Str() + string( " - " ) + tm2.Str() );
	
	for (Stime i=tm1; i<=tm2; ++i)
		ReportPeriodAllTraffic( i, i, out );
}

bool TrafSort( UserStat st1, UserStat st2 )
{
	return ( st1.tx > st2.tx );	
}

int ReportPeriodAllTraffic( Stime& tm1, Stime& tm2, ostream& out )
{
	DataBase db;
		
	UserStat st, tmp_st, sum;
	vector<UserStat> stat;
	vector<User> users;
	
	int res;
	
	stat.clear();
	db.GetUserList( users );
	// ip = 0.0.0.0 is user to say that we will output TOTAL traffic
	sum.ip.Assign( "0.0.0.0" ); sum.rx = 0; sum.tx = 0;
	
	for ( size_t i=0; i<users.size(); i++ ){
		st.ip = users[i].ip;	st.rx = 0;	st.tx = 0;
		
		for ( Stime j=tm1; j<=tm2; ++j ){
			res = db.GetStat( j, st.ip, tmp_st );
			if ( res != 1 && res != 5 ) { st.rx += tmp_st.rx; st.tx += tmp_st.tx; }
		}
		
		stat.push_back( st );
		sum.rx += st.rx;
		sum.tx += st.tx;
	}
	
	sort( stat.begin(), stat.end(), TrafSort );
	stat.push_back( sum );
	
	Report( Info, "DATE: " );
	if ( tm1 == tm2 ) Report( Info, tm1.Str() ); else Report( Info, tm1.Str() + string( " - " ) + tm2.Str() );
	
	Table<UserStat> tb( StatStrings, &out );
	vector<string> Label;
	
	tb.CalcColWidth( stat );
	Label.push_back( "Name" ); Label.push_back( "Tx" );	Label.push_back( "Rx" );
	tb.SetLabels( Label );
	
	tb.DrawLabels();
	for (size_t i=0; i<stat.size(); i++)
		tb.DispRow(stat[i]);
	
	return 0;
}


