/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Stime.h"
#include <ctime>
#include <cctype>
#include <sstream>

int Stime::Refresh()
{
	time_t a;
	tm* b = new tm;
	
	time(&a);
	b = localtime(&a);
	
	char text[32];
	stringstream ss;
	
	strftime(text, 32, "%d", b);
	ss << text << " ";
	strftime(text, 32, "%m", b);
	ss << text << " ";
	strftime(text, 32, "%Y", b);
	ss << text;
	
	ss >> D >> M >> Y;
	
	return 0;
	
}

string Stime::Str() const
{
	ostringstream ss;
	if (D<10) ss << "0" << D; else ss << D;
	ss << ".";
	if (M<10) ss << "0" << M; else ss << M;
	ss << "." << Y;
	return ss.str();
}

bool Stime::Assign( string s )
{
	string t;
	int num;
	
	for (int i=0; i<2; i++){
		
		if (s.find('.') == string::npos) return false;
		
		t = s.substr(0, s.find('.'));
		//cout << "t = " << t << endl;
		if (t.length()>2 || t.length() == 0) return false;
		
		num = 0;
		for (size_t j=0; j<t.length(); j++)
			if ( !isdigit( t[j] ) ) return false; else num = num*10 + (t[j] - '0');
		
		if (i==0){
			if (num<1 || num>31) return false;
		} else
			if (i==1){
				if (num<1 || num>12) return false;
			}
		
		if (i==0) D = num; else
		if (i==1) M = num;
		
		s.erase(0, s.find('.') + 1);
		//cout << "s = " << s << endl;
		
	}
	
	if (s.length()>4 || s.length() < 1) return false;
		
	num = 0;
	for (size_t j=0; j<s.length(); j++)
		if ( !isdigit( s[j] ) ) return false; else num = num*10 + (s[j] - '0');
	
	if (num<0 || num>9999) return false;
	Y = num;
	
	return true;
}

bool AssignPeriod( string s, Stime& T1, Stime& T2 )
{
	if (s.find('-') == string::npos) return false;
	
	string t;
	
	t = s.substr(0, s.find('-'));
	s.erase(0, s.find('-') + 1);
	
	return (T1.Assign( t ) && T2.Assign( s ) );
} 

string GetTime()
{
	time_t a;
	tm* b = new tm;
	
	time(&a);
	b = localtime(&a);
	
	char text[32];
	stringstream ss;
	
	strftime(text, 32, "%H", b);
	ss << text << ":";
	strftime(text, 32, "%M", b);
	ss << text << ":";
	strftime(text, 32, "%S", b);
	ss << text;
		
	return ss.str();
}

Stime& Stime::operator++()
{
	D++;
	if (D>31) {D=1; M++;}
	if (M>12) {M=1; Y++;}
	return *this;
}

bool operator!=( const Stime& tm1, const Stime& tm2 )
{
	return ( tm1.GetD() != tm2.GetD() || tm1.GetM() != tm2.GetM() || tm1.GetY() != tm2.GetY() );
}

bool operator==( const Stime& tm1, const Stime& tm2 )
{
	return ( tm1.GetD() == tm2.GetD() && tm1.GetM() == tm2.GetM() && tm1.GetY() == tm2.GetY() );
}

bool operator<( const Stime& tm1, const Stime& tm2 )
{
	if ( tm1.GetY() < tm2.GetY() ) return true; else
		if (tm1.GetY() > tm2.GetY() ) return false; else
			if ( tm1.GetM() < tm2.GetM() ) return true; else
				if (tm1.GetM() > tm2.GetM() ) return false; else
					return ( tm1.GetD() < tm2.GetD() );
}

bool operator<=( const Stime& tm1, const Stime& tm2 )
{
	return ( tm1 < tm2 || tm1 == tm2 );
}

