/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef _TABLE_
#define _TABLE_

#include <iostream>
#include <vector>
#include <string>

using namespace std;

template<class T> class Table{
private:
	int ColCount;
	ostream* out;
	vector<int> ColWidth;
	vector<string> Label;
	void( *Strings )( T&, vector<string>& );
	
	int Line( char ch, int n );
	int SepLine();		
public:
	Table( void( *aStrings )( T&, vector<string>&), ostream* );
	~Table();
	
	int CalcColWidth( vector< T >& );
	int SetLabels( vector< string >& );
	int DispRow( T& );
	int DrawLabels();
};

template<class T> Table<T>::Table( void( *a_Strings )( T&, vector<string>&), ostream* a_out)
{
	ColCount = 0;
	out = a_out;
	Strings = a_Strings;
	ColWidth.clear();
}

template<class T> Table<T>::~Table()
{
	out->~ostream();
}

template<class T> int Table<T>::Line( char ch, int n )
{
	for (int i=0; i<n; i++) *out << ch;
	return 0;
}

template<class T> int Table<T>::SepLine()
{
	*out << '+';
	for (size_t i=0; i<ColCount; i++){
		Line( '*', ColWidth[i] );
		*out << '+';
	}
	*out << endl;
	return 0;
}

template<class T> int Table<T>::CalcColWidth( vector<T>& cont )
{
	vector<string> Word;
	
	if ( cont.size() == 0 ) return 1;
	
	ColWidth.clear();
	Strings( cont[0], Word );
	ColCount = Word.size();
	for (size_t i=0; i<ColCount; i++) ColWidth.push_back( (int)Word[i].length() );
	
	for( size_t i=1; i<cont.size(); i++ ){
		Strings( cont[i], Word );
		for (size_t i=0; i<ColCount; i++)
			if ( Word[i].length() > ColWidth[i] )
				ColWidth[i] = Word[i].length();
	}
	
	for (int i=0; i<ColCount; i++) ColWidth[i] += 2;
	
	return 0;
}

template<class T> int Table<T>::DispRow( T& sct )
{
	int pref;
	vector<string> Word;
	
	Strings( sct, Word );
		
	for (size_t i=0; i<Word.size(); i++){	
		pref = ( ColWidth[i] - Word[i].length() ) / 2;
		*out << '|';
		Line( ' ', pref );
		*out << Word[i];
		Line( ' ', ( ColWidth[i] - Word[i].length() ) - pref );
	}
	
	*out << '|' << endl;
	SepLine();
	
	return 0;
}

template<class T> int Table<T>::SetLabels( vector<string>& a_Label )
{
	if ( a_Label.size() != ColCount ) return 1;
	
	Label.clear();
	for (size_t i=0; i<ColCount; i++){
		Label.push_back( a_Label[i] );
		if ( a_Label[i].length() > ColWidth[i] ) ColWidth[i] = a_Label[i].length() + 2;
	}
	
	return 0;
}

template<class T> int Table<T>::DrawLabels()
{
	int pref;
	
	SepLine();
	for (int i=0; i<ColCount; i++){
		pref = ( ColWidth[i] - Label[i].length() ) / 2;
		*out << '|';
		Line( ' ', pref );
		*out << Label[i];
		Line( ' ', ( ColWidth[i] - Label[i].length() ) - pref );
	}
	*out << '|' << endl;
	SepLine();
	
	return 0;
}

#endif
