/***************************************************************************
 *   Copyright (C) 2008 by potlaty   *
 *   potlaty@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _USERCONTAINER_
#define _USERCONTAINER_

#include <fstream>
#include <vector>
#include <string>
#include "user.h"
#include "defs.h"
#include "IP.h"

using namespace std;

template<class T> class UserContainer{
private:
	vector<T> users;
public:
	int Load(const string&);
	int Store(const string&);
	int Insert(const T&);
	int Delete(const int n);
	int Clear();
	size_t Size();
	
	int Find(const IP& ip, T&);
	int Find(const string& login, T&);
	
	T operator[] (const int n);
};

template<class T> int UserContainer<T>::Load(const string& fname)
{
	T t;
	int n;
	string s;
	ifstream in;
	
	in.open(fname.c_str(), ios_base::in);
	if (!in) return 1;
	
	in >> n;
	getline(in, s);
	for (int i=0; i<n; i++){
		in >> t;		
		users.push_back(t);
	}
	
	in.close();
	
	return 0;
}

template<class T> int UserContainer<T>::Store(const string& fname)
{
	T t;
	ofstream out;
	
	out.open(fname.c_str(), ios_base::out);
	if (!out) return 2;
	
	out << users.size() << endl;
	typename vector<T>::iterator VTI;
	
	for (VTI=users.begin(); VTI!=users.end(); VTI++)	out << *VTI;
	
	out.close();
	
	return 0;
}

template<class T> int UserContainer<T>::Insert(const T& user)
{
	bool found = false;
	typename vector<T>::iterator VTI;
	for (VTI = users.begin(); VTI != users.end(); VTI++)
		if (VTI->ip == user.ip) {found = true; break;}
	
	if ( found ) *VTI = user; else users.push_back(user);
	return 0;
}

template<class T> int UserContainer<T>::Delete(const int n)
{
	if (n<0 || n>=Size()) return 4;
	
	typename vector<T>::iterator VTI = users.begin();
	for (int i=0; i<n; i++) VTI++;
	users.erase(VTI);
	return 0;
}

template<class T> size_t UserContainer<T>::Size()
{
	return users.size();
}

template<class T> T UserContainer<T>::operator[] (const int n)
{
	return users[n];
}

template<class T> int UserContainer<T>::Clear()
{
	users.clear();
	return 0;
}

template<class T> int UserContainer<T>::Find(const IP& ip, T& user)
{
	typename vector<T>::iterator VTI;
	for (VTI=users.begin(); VTI!=users.end(); VTI++)
		if (VTI->ip == ip){ 
			user = *VTI; 
			return 0;
		}
	return 5;
}

template<class T> int UserContainer<T>::Find(const string& login, T& user)
{
	typename vector<T>::iterator VTI;
	for (VTI=users.begin(); VTI!=users.end(); VTI++)
		if (VTI->login == login){ 
			user = *VTI; 
			return 0;
		}
	return 5;
}

#endif
